%% THIS IS AN M-FILE THAT DEMONSTRATES MATLAB(R)'S CAPABILITY IN IMAGE AQUISITION AND ANALYSIS
%% 0. CLOSE ALL FIGURES AND CLEAR VARIABLES BEFORE YOU START!
%This demonstrates how to carry out bacterial colony counting, which is a
%significant technical hurdle for vaccine studies as well as various
%microbiological studies. MATLAB can be used to process images obtained by
%a digital camera or document scanner.
% The image shows a square Petri dish with pneumococcal colonies. The
% colonies are in 24 %spots, arranged in 3 columns and 8 rows. The image
% was obtained with a document scanner as described in the following
% refernce. The number of bacterial colonies in each spot is shown in the
% table with the figure. Reference: Putman, M, R. Burton, and M.H. Nahm.
% "Simplified method to automatically count bacterial colony forming
% unit". Journal of Immunological Methods, 302 (2005) 99 102.
close all; clear all;

%% 1. READ THE IMAGE FILE!
cfu=imread('D:\Program Files\MATLAB\CFU','bmp');

%% 2. CONVERSION FROM TRUECOLOR TO GRAYSCALE IMAGE!
% I=rgb2gray(cfu)converts the truecolor image cfu to the grayscale
% intensity image, I. It eliminates the hue and saturation information
% while retains the luminance.
I = rgb2gray(cfu);

%% 3. FIRST MORPHOLOGICAL IMAGE TREATMENT: CLOSE OPERATION (OBJECT INTERCONNECTION)!
% SE = strel('disk', R, N) creates a flat, disk-shaped structuring element,
% where R specifies the radius.  R must be a nonnegative integer.  N must
% be 0, 4, 6, or 8.  When N is greater than 0, the disk-shaped structuring
% element is approximated by a sequence of N periodic-line structuring
% elements.  When N equals 0, no approximation is used, and the structuring
% element members consist of all pixels whose centers are no greater than R
% away from the origin. If N is not specified, the default value will be 4.
% Specify a radius of one pixel so that objects with radius 1 pixel or
% larger will be accounted for.  See Section 4.7 for further information on
% imclose function.
SE = strel('disk', 1, 4);
bckgrnd=imclose(I,SE);
figure(1); imshow(bckgrnd);

%% 4. SECOND MORPHOLOGICAL TREATMENT: REMOVAL OF BACKGROUND LUMINANCE!
% I2 = imsubtract(bckgrnd,I) subtracts each element in array I from the 
% corresponding element in array bckgrnd and returns the difference in 
% the corresponding element of the output array I2
I2=imsubtract(bckgrnd,I);
% figure(2);imshow(I2,[]); %no need to show the figure; 

%% 5. SETTING A THRESHOLD FOR LIGHT INTENSITY (LUMINANCE)!
% First, level= graythresh(I2) computes a global threshold (level between 0
% and 1) that can be used to convert an intensity image to a binary image
% with im2bw.
level=graythresh(I2);

%% 6. CONVERSION OF GRAY IMAGE INTO BINARY IMAGE WHILE UTILIZING THE THRESHOLD!
% Second, BW = im2bw(I2, graythresh(I2)) converts the grayscale image I2 to
% a binary image, BW. The output image BW replaces all pixels in the input
% image with luminance greater than level with the value 1 (white) and
% replaces all other pixels with the value 0 (black).
BW=im2bw(I2,level);

%% 7. THIRD MORPHOLOGICAL TREATMENT: REMOVAL OF ARTIFACTS (SMALL OBJECTS LESS THAN 2 PIXELS IN SIZE)!
% BW2=bwareaopen(BW,3) Removes from a binary image, BW, all connected 
% components (objects) that have fewer than 2 pixels, producing another 
% binary image, BW2. The default connectivity is 8 for two dimensions.
BW2=bwareaopen(BW,2);
figure(3); imshow(BW2);

%% 8. FOURTH MORPHOLOGICAL TREATMENT: REMOVAL OF BORDER ARTIFACTS (CONNECTED TO THE IMAGE BORDER)!
% BW3 = imclearborder(BW2) suppresses structures that are both lighter than their
% surroundings and are connected to the image border. (In other words, use
% this function to clear the image border.) BW2 can be a grayscale or
% binary image. The output image, BW3, is grayscale or binary,
% respectively. The default connectivity is 8 for two dimensions.
BW3=imclearborder(BW2);
figure(4); imshow(BW3);

%% 9. FIFTH MORPHOLOGICAL TREATMENT: FILLING IN THE GAPS WITHIN OBJECTS!
% fill=imfill(BW3,'holes') fills holes in the binary image BW3. A hole is 
% a set of background pixels that cannot be reached by filling in the backg-
% round from the edge of the image.
fill=imfill(BW3,'holes');
%figure(5); imshow(fill);  %no need to show the figure; 

%% 10. COUNT THE NUMBER OF CONNECTED OBJECTS!
% ConComp = bwconncomp(fill) returns the connected components, ConComp,
% found in THE FINAL, FINISHED BINARY IMAGE, fill. The binary image, fill,
% can have any dimension. ConComp is a structure with four fields. One
% important field is number of objects, NumObjects.
[ConComp]=bwconncomp(fill);

%% 11. DETERMINE OBJECT'S GEOMETRICAL AND SHPAE PROPERTIES!
% STATS = regionprops(ConComp, properties) measures a set of properties for
% each connected component (object) in ConComp, which is a structure
% returned by bwconncomp. Examples of properties are: Area, BoundingBox,
% Centroid, Eccentricity, EquivDiameter, Extent, MajorAxisLength,
% MinorAxisLength, and Perimeter. Please, see regionprops using online help.
Stats=regionprops(ConComp,'EquivDiameter');

%% 12. REDEFINING VARIABLES FOR SIMPLICITY!
% STATS is a structure array with length equal to the number of objects:
% ConComp.NumObjects. The fields of the structure array denote different
% properties for each region, as specified by properties.
% EqDiam=[Stats.EquivDiameter] extracts the Stats's field, that is, 
% EquivDiameter into a separate vector, EqDiam. EqDiam has one value for 
% each object.
EqDiam=[Stats.EquivDiameter];
%% 13. DEFINE MEAN EQUIVALENT DIAMETER AND STANDARD DEVIATION.
% A restriction to accept CFU is to have its EQUIVALENT DIAMETER FALL
% WITHING THE RANGE SPCEFIED BY THE CONDITION BELOW.
MEqDiam=mean(EqDiam);
Stdev=std(EqDiam);
%% 14.CFU WITH EITHER TOO SMALL OR TOO LARGE EQUIVALENT DIAMTER WILL BE REJECTED.
%If the equivalent diameter (EqDiam) of CFU is found to be too large or too
%small, compared with the mean equivalent diameter of all found objects,
%then this CFU will be rejected (i.e., considered an artifact).
%IndexOfReject is the index of the matrix element returned by find
%statement once the condition is met or fulfilled.
%StatReject=Stats(IndexOfReject) picks up only the rejected objects.
IndexOfReject=find(EqDiam>(MEqDiam+2.0*Stdev) | EqDiam<(MEqDiam-2.0*Stdev));
StatReject=Stats(IndexOfReject);

%% 15.  COUNTING # OF CFUs (COLONIES) 
% Counting Colony Forming Unit Objects
% is done via subtracting the number of rejected objects from the total
% count of candidate objects.  Keep in mind that an artifact is either too
% small or too large, compared with the mean value. A 95% confidence
% interval (i.e., 2sigma) is taken around the mean value of the equivalent
% diameter of all found CFU objects.

TotalNum=ConComp.NumObjects;
display (['The total number of candidate CFUs: ',num2str(TotalNum)]);
CFUReject=length(StatReject);
display (['The number of rejected (too small or too large)CFUs is: ', ...
    num2str(CFUReject)]);
CFUNUM=ConComp.NumObjects-length(StatReject);
display (['The number of accepted CFUs is: ',num2str(CFUNUM)]);
